package com.dayou.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.configuration.BaseEntity;
import com.dayou.configuration.RestResponse;
import com.dayou.configuration.annotation.IgnoreAuth;
import com.dayou.dto.CommitDTO;
import com.dayou.dto.DocumentDTO;
import com.dayou.entity.Document;
import com.dayou.service.IDocumentService;
import com.dayou.service.IProfessorResultService;
import com.dayou.vo.DocumentCycleVO;
import com.dayou.vo.DocumentVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/13
 * created with IntelliJ IDEA.
 */
@RestController
@RequestMapping("/document")
@Slf4j
public class DocumentController {

    @Autowired
    private IDocumentService documentService;

    @Autowired
    private IProfessorResultService professorResultService;

    /**
     * 获取当前可用的问卷文档
     * @return
     */
    @IgnoreAuth
    @GetMapping("/current")
    public RestResponse<DocumentCycleVO> getDocument(){
        DocumentCycleVO documentCycleVO = documentService.getDocumentCycleVO();
        return RestResponse.data(documentCycleVO);
    }

    /**
     * 提交问卷
     * @return
     */
    @IgnoreAuth
    @PostMapping("/commit")
    public RestResponse<Boolean> commitDocument(@RequestBody @Valid DocumentCycleVO documentCycleVO){
        Boolean result = professorResultService.commitDocument(documentCycleVO);
        return RestResponse.data(result);
    }

    /**
     * 问卷列表
     * @param document
     * @param page
     * @return
     */
    @GetMapping("")
    public RestResponse<IPage<DocumentVO>> getPage(Page page, Document document ){
        IPage<DocumentVO> result = documentService.getPage(page,document);
        return RestResponse.data(result);
    }

    /**
     * 新增问卷
     * @param document
     * @return
     */
    @PostMapping("")
    public RestResponse<Boolean> saveDocument(@RequestBody Document document){
        Boolean result = documentService.saveOrUpdate(document);
        return RestResponse.data(result);
    }
}
