package com.dayou.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.configuration.BaseEntity;
import com.dayou.configuration.RestResponse;
import com.dayou.entity.Question;
import com.dayou.service.IQuestionService;
import com.dayou.utils.ConvertUtil;
import com.dayou.vo.LabelKeyVO;
import com.dayou.vo.QuestionTreeVO;
import com.dayou.vo.QuestionVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/20
 * created with IntelliJ IDEA.
 */
@RestController
@RequestMapping("/question")
@Slf4j
public class QuestionController {

    @Autowired
    private IQuestionService questionService;

    @GetMapping("")
    public RestResponse<IPage<QuestionVO>> getQuestion(Question question, Page page){
        Page xPage = questionService.page(page, new LambdaQueryWrapper<Question>()
                .like(StrUtil.isNotBlank(question.getLabel()), Question::getLabel, question.getLabel()));
        return RestResponse.data(xPage);
    }

    @PostMapping("")
    public RestResponse<Long> saveQuestion(@RequestBody @Valid Question question){
        boolean b = questionService.saveOrUpdate(question);
        return RestResponse.data(question.getId());
    }

    @GetMapping("/simpleAll")
    public RestResponse<List<LabelKeyVO>> getList(){
        List<Question> list = questionService.list(new LambdaQueryWrapper<Question>().select(BaseEntity::getId, Question::getLabel).isNull(Question::getParentId));
        List<LabelKeyVO> labelKeyVOS = ConvertUtil.copyList(list, LabelKeyVO.class);
        labelKeyVOS.stream().forEach(x->{
            x.setKey(x.getId());
            x.setLabel(x.getLabel());
        });
        return RestResponse.data(labelKeyVOS);
    }

    /**
     * 题目树
     * @param question
     * @return
     */
    @GetMapping("/tree")
    public RestResponse<List<QuestionTreeVO>> getQuestionTree(Question question){
        List<QuestionTreeVO> tree = questionService.getQuestionTree(question);
        return RestResponse.data(tree);
    }

    @DeleteMapping("/{id}")
    public RestResponse<Boolean> remove(@PathVariable("id") Long id){
        return RestResponse.data(questionService.removeById(id));
    }
}
