package com.dayou.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.dao.QuestionAnalysisMapper;
import com.dayou.entity.QuestionAnalysis;
import com.dayou.service.IQuestionAnalysisService;
import com.dayou.utils.Tree;
import com.dayou.utils.TreeUtil;
import com.dayou.vo.AnalysisResultVO;
import com.dayou.vo.ChildrenParentName;
import com.dayou.vo.MergeTableVO;
import com.dayou.vo.MergeVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/19
 * created with IntelliJ IDEA.
 */
@Service
public class QuestionAnalysisServiceImpl extends ServiceImpl<QuestionAnalysisMapper, QuestionAnalysis> implements IQuestionAnalysisService {

    @Autowired
    private QuestionAnalysisMapper questionAnalysisMapper;

    @Override
    public List<AnalysisResultVO> getAnalysisList() {
        List<AnalysisResultVO> analysisList = questionAnalysisMapper.getAnalysisList();
        analysisList = (List<AnalysisResultVO>) TreeUtil.buildTree(analysisList,analysisList.get(0).getTypeId());
        return analysisList;
    }

    @Override
    public List<AnalysisResultVO> getAnalysisListByCycleId(Long id) {
        return questionAnalysisMapper.getAnalysisListByCycleId(id);
    }

    @Override
    public MergeVO getMergeData() {
        MergeVO merge = new MergeVO();
        List<ChildrenParentName> parentLabel = questionAnalysisMapper.getParentLabel();
        Map<String, String> childrenParentMap = parentLabel.stream().collect(Collectors.toMap(ChildrenParentName::getLabel, ChildrenParentName::getParentLabel));
        List<AnalysisResultVO> analysisList = questionAnalysisMapper.getAnalysisList();
        merge.setItemName(analysisList.get(0).getItemName());
        merge.setCycleName(analysisList.get(0).getCycleName());
        merge.setTypeName(analysisList.get(0).getTypeName());
        if (analysisList.get(0).getQuestionId()==null){
            return merge;
        }
        Map<String, BigDecimal> labelScore = analysisList.stream().collect(Collectors.toMap(AnalysisResultVO::getQuestionName, AnalysisResultVO::getAvgScore));
        analysisList = (List<AnalysisResultVO>) TreeUtil.buildTree(analysisList,analysisList.get(0).getTypeId());
        List<AnalysisResultVO> allChildren = Lists.newArrayList();
        for (AnalysisResultVO analysisResultVO : analysisList){
                allChildren = getChildrenIsNull(analysisResultVO, allChildren);
        }
        List<MergeTableVO> result = Lists.newArrayList();
        for (AnalysisResultVO analysisResultVO : allChildren){
            if (analysisResultVO.getScope().equals("名称")){
                MergeTableVO mergeVO = MergeTableVO.builder()
                        .name(analysisResultVO.getQuestionName()
                                +"-"+ labelScore.get(analysisResultVO.getQuestionName())).build();
                result.add(mergeVO);
            }
            if (analysisResultVO.getScope().equals("定级因素")){
                MergeTableVO mergeVO = MergeTableVO.builder().element(analysisResultVO.getQuestionName()
                                +"-"+ labelScore.get(analysisResultVO.getQuestionName()) )
                        .name(childrenParentMap.get(analysisResultVO.getQuestionName())
                                +"-"+ labelScore.get(childrenParentMap.get(analysisResultVO.getQuestionName()))).build();
                result.add(mergeVO);
            }
            if (analysisResultVO.getScope().equals("一级因子")){
                MergeTableVO mergeVO = MergeTableVO.builder().element1(analysisResultVO.getQuestionName()
                                +"-"+ labelScore.get(analysisResultVO.getQuestionName()))
                        .element(childrenParentMap.get(analysisResultVO.getQuestionName())
                                +"-"+ labelScore.get(childrenParentMap.get(analysisResultVO.getQuestionName())))
                        .name(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName()))
                                +"-"+ labelScore.get(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName())))).build();
                result.add(mergeVO);
            }
            if (analysisResultVO.getScope().equals("二级因子")){
                MergeTableVO mergeVO = MergeTableVO.builder().element2(analysisResultVO.getQuestionName()
                                +"-"+ labelScore.get(analysisResultVO.getQuestionName()))
                        .element1(childrenParentMap.get(analysisResultVO.getQuestionName())
                                +"-"+ labelScore.get(childrenParentMap.get(analysisResultVO.getQuestionName())))
                        .element(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName()))
                                +"-"+ labelScore.get(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName()))))
                        .name(childrenParentMap.get(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName())))
                                +"-"+ labelScore.get(childrenParentMap.get(childrenParentMap.get(childrenParentMap.get(analysisResultVO.getQuestionName()))))).build();
                result.add(mergeVO);
            }
        }
        merge.setAnalysisData(result);
        return merge;
    }

    private List<AnalysisResultVO> getChildrenAll(AnalysisResultVO analysisResultVO,List<AnalysisResultVO> allChildren){
        List<AnalysisResultVO> children = analysisResultVO.getChildren();
        if (CollectionUtils.isEmpty(children)){
            return allChildren;
        }
        allChildren.addAll(children);
        for (AnalysisResultVO analysis : children){
            getChildrenAll(analysis,allChildren);
        }
        return null;
    }

    private List<AnalysisResultVO> getChildrenIsNull(AnalysisResultVO analysisResultVO,List<AnalysisResultVO> childrenNull){
        List<AnalysisResultVO> children = analysisResultVO.getChildren();
        if (CollectionUtils.isEmpty(children)){
            childrenNull.add(analysisResultVO);
        }else {
            for (AnalysisResultVO analysis : children){
                getChildrenIsNull(analysis,childrenNull);
            }
        }
        return childrenNull;
    }
}
